@@ -1,54 +1,44 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtLocation module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 #include "qgeocameratiles_p.h"
-
 #include "qgeocameradata_p.h"
-#include "qgeoprojection_p.h"
 #include "qgeotilespec_p.h"
-
-#include "qdoublevector2d_p.h"
-#include "qdoublevector3d_p.h"
-
 #include "qgeomaptype_p.h"
 
+#include <QtPositioning/private/qgeoprojection_p.h>
+#include <QtPositioning/private/qdoublevector2d_p.h>
+#include <QtPositioning/private/qdoublevector3d_p.h>
+
 #include <QVector>
 #include <QMap>
 #include <QPair>
@@ -77,7 +67,7 @@ struct Frustum
     QDoubleVector3D bottomRightFar;
 };
 
-typedef QVector<QDoubleVector3D> Polygon;
+typedef QVector<QDoubleVector3D> PolygonVector;
 
 class QGeoCameraTilesPrivate
 {
@@ -87,6 +77,7 @@ public:
 
     QString pluginString_;
     QGeoMapType mapType_;
+    int mapVersion_;
     QGeoCameraData camera_;
     QSize screenSize_;
     int tileSize_;
@@ -112,13 +103,13 @@ public:
     };
 
     void appendZIntersects(const QDoubleVector3D &start, const QDoubleVector3D &end, double z, QVector<QDoubleVector3D> &results) const;
-    Polygon frustumFootprint(const Frustum &frustum) const;
+    PolygonVector frustumFootprint(const Frustum &frustum) const;
 
-    QPair<Polygon, Polygon> splitPolygonAtAxisValue(const Polygon &polygon, int axis, double value) const;
-    QPair<Polygon, Polygon> clipFootprintToMap(const Polygon &footprint) const;
+    QPair<PolygonVector, PolygonVector> splitPolygonAtAxisValue(const PolygonVector &polygon, int axis, double value) const;
+    QPair<PolygonVector, PolygonVector> clipFootprintToMap(const PolygonVector &footprint) const;
 
     QList<QPair<double, int> > tileIntersections(double p1, int t1, double p2, int t2) const;
-    QSet<QGeoTileSpec> tilesFromPolygon(const Polygon &polygon) const;
+    QSet<QGeoTileSpec> tilesFromPolygon(const PolygonVector &polygon) const;
 
     struct TileMap
     {
@@ -247,6 +238,17 @@ void QGeoCameraTiles::setMapType(const QGeoMapType &mapType)
     d->updateMetadata();
 }
 
+void QGeoCameraTiles::setMapVersion(const int mapVersion)
+{
+    Q_D(QGeoCameraTiles);
+
+    if (d->mapVersion_ == mapVersion)
+        return;
+
+    d->mapVersion_ = mapVersion;
+    d->updateMetadata();
+}
+
 void QGeoCameraTiles::setTileSize(int tileSize)
 {
     Q_D(QGeoCameraTiles);
@@ -284,10 +286,9 @@ QSet<QGeoTileSpec> QGeoCameraTiles::tiles() const
 }
 
 QGeoCameraTilesPrivate::QGeoCameraTilesPrivate()
-    : tileSize_(0),
-      maxZoom_(0),
-      intZoomLevel_(0),
-      sideLength_(0) {}
+:   mapVersion_(-1), tileSize_(0), maxZoom_(0), intZoomLevel_(0), sideLength_(0)
+{
+}
 
 QGeoCameraTilesPrivate::~QGeoCameraTilesPrivate() {}
 
@@ -302,7 +303,7 @@ void QGeoCameraTilesPrivate::updateMetadata()
 
     for (; i != end; ++i) {
         QGeoTileSpec tile = *i;
-        newTiles.insert(QGeoTileSpec(pluginString_, mapType_.mapId(), tile.zoom(), tile.x(), tile.y()));
+        newTiles.insert(QGeoTileSpec(pluginString_, mapType_.mapId(), tile.zoom(), tile.x(), tile.y(), mapVersion_));
     }
 
     tiles_ = newTiles;
@@ -315,10 +316,10 @@ void QGeoCameraTilesPrivate::updateGeometry(double viewExpansion)
     Frustum f = frustum(viewExpansion);
 
     // Find the polygon where the frustum intersects the plane of the map
-    Polygon footprint = frustumFootprint(f);
+    PolygonVector footprint = frustumFootprint(f);
 
     // Clip the polygon to the map, split it up if it cross the dateline
-    QPair<Polygon, Polygon> polygons = clipFootprintToMap(footprint);
+    QPair<PolygonVector, PolygonVector> polygons = clipFootprintToMap(footprint);
 
     if (!polygons.first.isEmpty()) {
         QSet<QGeoTileSpec> tilesLeft = tilesFromPolygon(polygons.first);
@@ -485,9 +486,9 @@ top:
 
 
 // Returns the intersection of the plane of the map and the camera frustum as a right handed polygon
-Polygon QGeoCameraTilesPrivate::frustumFootprint(const Frustum &frustum) const
+PolygonVector QGeoCameraTilesPrivate::frustumFootprint(const Frustum &frustum) const
 {
-    Polygon points;
+    PolygonVector points;
     points.reserve(24);
 
     appendZIntersects(frustum.topLeftNear, frustum.topLeftFar, 0.0, points);
@@ -525,7 +526,7 @@ Polygon QGeoCameraTilesPrivate::frustumFootprint(const Frustum &frustum) const
     //   - start with the first point, put it in the sorted part of the list
     //   - add the nearest unsorted point to the last sorted point to the end
     //     of the sorted points
-    Polygon::iterator i;
+    PolygonVector::iterator i;
     for (i = points.begin(); i != points.end(); ++i) {
         sorter.base = *i;
         if (i + 1 != points.end())
@@ -551,15 +552,15 @@ Polygon QGeoCameraTilesPrivate::frustumFootprint(const Frustum &frustum) const
     return points;
 }
 
-QPair<Polygon, Polygon> QGeoCameraTilesPrivate::splitPolygonAtAxisValue(const Polygon &polygon, int axis, double value) const
+QPair<PolygonVector, PolygonVector> QGeoCameraTilesPrivate::splitPolygonAtAxisValue(const PolygonVector &polygon, int axis, double value) const
 {
-    Polygon polygonBelow;
-    Polygon polygonAbove;
+    PolygonVector polygonBelow;
+    PolygonVector polygonAbove;
 
     int size = polygon.size();
 
     if (size == 0) {
-        return QPair<Polygon, Polygon>(polygonBelow, polygonAbove);
+        return QPair<PolygonVector, PolygonVector>(polygonBelow, polygonAbove);
     }
 
     QVector<int> comparisons = QVector<int>(polygon.size());
@@ -638,11 +639,11 @@ QPair<Polygon, Polygon> QGeoCameraTilesPrivate::splitPolygonAtAxisValue(const Po
         }
     }
 
-    return QPair<Polygon, Polygon>(polygonBelow, polygonAbove);
+    return QPair<PolygonVector, PolygonVector>(polygonBelow, polygonAbove);
 }
 
 
-QPair<Polygon, Polygon> QGeoCameraTilesPrivate::clipFootprintToMap(const Polygon &footprint) const
+QPair<PolygonVector, PolygonVector> QGeoCameraTilesPrivate::clipFootprintToMap(const PolygonVector &footprint) const
 {
     bool clipX0 = false;
     bool clipX1 = false;
@@ -651,7 +652,7 @@ QPair<Polygon, Polygon> QGeoCameraTilesPrivate::clipFootprintToMap(const Polygon
 
     double side = 1.0 * sideLength_;
 
-    typedef Polygon::const_iterator const_iter;
+    typedef PolygonVector::const_iterator const_iter;
 
     const_iter i = footprint.constBegin();
     const_iter end = footprint.constEnd();
@@ -667,7 +668,7 @@ QPair<Polygon, Polygon> QGeoCameraTilesPrivate::clipFootprintToMap(const Polygon
             clipY1 = true;
     }
 
-    Polygon results = footprint;
+    PolygonVector results = footprint;
 
     if (clipY0) {
         results = splitPolygonAtAxisValue(results, 1, 0.0).second;
@@ -681,9 +682,9 @@ QPair<Polygon, Polygon> QGeoCameraTilesPrivate::clipFootprintToMap(const Polygon
         if (clipX1) {
             results = splitPolygonAtAxisValue(results, 0, 0.0).second;
             results = splitPolygonAtAxisValue(results, 0, side).first;
-            return QPair<Polygon, Polygon>(results, Polygon());
+            return QPair<PolygonVector, PolygonVector>(results, PolygonVector());
         } else {
-            QPair<Polygon, Polygon> pair = splitPolygonAtAxisValue(results, 0, 0.0);
+            QPair<PolygonVector, PolygonVector> pair = splitPolygonAtAxisValue(results, 0, 0.0);
             if (pair.first.isEmpty()) {
                 // if we touched the line but didn't cross it...
                 for (int i = 0; i < pair.second.size(); ++i) {
@@ -719,7 +720,7 @@ QPair<Polygon, Polygon> QGeoCameraTilesPrivate::clipFootprintToMap(const Polygon
         }
     } else {
         if (clipX1) {
-            QPair<Polygon, Polygon> pair = splitPolygonAtAxisValue(results, 0, side);
+            QPair<PolygonVector, PolygonVector> pair = splitPolygonAtAxisValue(results, 0, side);
             if (pair.second.isEmpty()) {
                 // if we touched the line but didn't cross it...
                 for (int i = 0; i < pair.first.size(); ++i) {
@@ -753,7 +754,7 @@ QPair<Polygon, Polygon> QGeoCameraTilesPrivate::clipFootprintToMap(const Polygon
             }
             return pair;
         } else {
-            return QPair<Polygon, Polygon>(results, Polygon());
+            return QPair<PolygonVector, PolygonVector>(results, PolygonVector());
         }
     }
 
@@ -793,7 +794,7 @@ QList<QPair<double, int> > QGeoCameraTilesPrivate::tileIntersections(double p1,
     return results;
 }
 
-QSet<QGeoTileSpec> QGeoCameraTilesPrivate::tilesFromPolygon(const Polygon &polygon) const
+QSet<QGeoTileSpec> QGeoCameraTilesPrivate::tilesFromPolygon(const PolygonVector &polygon) const
 {
     int numPoints = polygon.size();
 
@@ -990,7 +991,7 @@ QSet<QGeoTileSpec> QGeoCameraTilesPrivate::tilesFromPolygon(const Polygon &polyg
         int minX = i->first;
         int maxX = i->second;
         for (int x = minX; x <= maxX; ++x) {
-            results.insert(QGeoTileSpec(pluginString_, mapType_.mapId(), z, x, y));
+            results.insert(QGeoTileSpec(pluginString_, mapType_.mapId(), z, x, y, mapVersion_));
         }
     }
 
